/*   Little C. */

#include "header.h"
#include "stucturs.h"

bool InitLittleC(const char *fileName)
{
    /*     */
    if((p_buf = (char *) malloc(PROG_SIZE))==NULL){cout<<"Cant use memory!"<<endl;return false;}
    /*     */
    if(!load_program(p_buf, fileName)){cout<<"Cant open file: "<<fileName; return false;}
    /*   long jump */
    if(setjmp(e_buf)){cout<<"Error buffer long jump"; return false;}
    gvar_index = 0;  /*     */

    /*        */
    prog = p_buf;

    prescan(); /*    
                    */

    lvartos = 0;     /*      */
    functos = 0;     /*     (CALL) */

    return true;
}

void StartLittleC()
{
    /*   main() */
    prog = find_func("main"); /*     */

    if(!prog) { /*  main()    */
    cout<<"main() not found!"<<endl;
    exit(1);
    }

    prog--; /*     ( */
    strcpy(token, "main");
    call(); /*   main() */
}

void ExectFunct(const char *functName)
{
    /*   main() */
    prog = find_func(functName); /*     */

    if(!prog) { /*  main()    */
    cout<<functName<<"() not found!"<<endl;
    exit(1);
    }

    prog--; /*     ( */
    strcpy(token, functName);
    call(); /*   main() */
}

/*     . 
   interp_block()    
   ,  main()  
        return.
*/
void interp_block()
{
  int value;
  char block = 0;

  do {
    token_type = get_token();

    /*    
           .
    */

    /*    */
    if(token_type == IDENTIFIER) {
      /*    ,
          . */
      putback();  /*    
            eval_exp() */
      eval_exp(&value);  /*   */
      if(*token!=';') sntx_err(SEMI_EXPECTED);
    }
    else if(token_type==BLOCK) {
      /*     */
      if(*token == '{') /*  */
        block = 1; /*  ,    */
      else return; /*  },  */
    }
    else /*   */
      switch(tok) {
        case CHAR_L:
        case INT_L:     /*    */
          putback();
          decl_local();
          break;
        case RETURN_L:  /*     */
          func_ret();
          return;
        case IF_L:      /*   if */
          exec_if();
          break;
        case ELSE_L:    /*   else */
          find_eob(); /*    else
                            */
          break;
        case WHILE_L:   /*   while */
          exec_while();
          break;
        case DO_L:      /*   do-while */
          exec_do();
          break;
        case FOR_L:     /*   for */
          exec_for();
          break;
        case END_L:
          exit(0);
      }
  } while (tok != FINISHED_L && block);
}

/*  . */
int load_program(char *p, const char *fname)
{
  FILE *fp;
  int i=0;

  if((fp=fopen(fname, "rb"))==NULL) return 0;

  i = 0;
  do {
    *p = getc(fp);
    p++; i++;
  } while(!feof(fp) && i<PROG_SIZE);

  if(*(p-2) == 0x1a) *(p-2) = '\0'; /*  
                                         */
  else *(p-1) = '\0';
  fclose(fp);
  return 1;
}

/*      
      . */
void prescan()
{
  char *p, *tp;
  char temp[32];
  int datatype;
  int brace = 0;  /*  brace = 0,  
                        
                       - . */

  p = prog;
  func_index = 0;
  do {
    while(brace) {  /*    */
      get_token();
      if(*token == '{') brace++;
      if(*token == '}') brace--;
    }

    tp = prog; /*    */
    get_token();
    /*        */
    if(tok==CHAR_L || tok==INT_L) {
      datatype = tok; /*    */
      get_token();
      if(token_type == IDENTIFIER) {
        strcpy(temp, token);
        get_token();
        if(*token != '(') { /*      */
          prog = tp; /*     */
          decl_global();
        }
        else if(*token == '(') {  /*     */
          func_table[func_index].loc = prog;
          func_table[func_index].ret_type = datatype;
          strcpy(func_table[func_index].func_name, temp);
          func_index++;
          while(*prog != ')') prog++;
          prog++;
          /*  prog   
                */
        }
        else putback();
      }
    }
    else if(*token == '{') brace++;
  } while(tok != FINISHED_L);
  prog = p;
}

/*      .
    NULL,   .
*/
char *find_func(const char *name)
{
  register int i;

  for(i=0; i < func_index; i++)
    if(!strcmp(name, func_table[i].func_name))
      return func_table[i].loc;

  return NULL;
 }

/*   . */
void decl_global()
{
  int vartype;

  get_token();  /*   */

  vartype = tok; /*    */

  do { /*   */
    global_vars[gvar_index].v_type = vartype;
    global_vars[gvar_index].value = 0;  /*   */
    get_token();  /*   */
    strcpy(global_vars[gvar_index].var_name, token);
    get_token();
    gvar_index++;
  } while(*token == ',');
  if(*token != ';') sntx_err(SEMI_EXPECTED);
}

/*   . */
void decl_local()
{
  struct var_type i;

  get_token();  /*   */

  i.v_type = tok;
  i.value = 0;  /*   */

  do { /*   */
    get_token(); /*    */
    strcpy(i.var_name, token);
    local_push(i);
    get_token();
  } while(*token == ',');
  if(*token != ';') sntx_err(SEMI_EXPECTED);
}

/*  . */
void call()
{
  char *loc, *temp;
  int lvartemp;

  loc = find_func(token); /*     */
  if(loc == NULL)
    sntx_err(FUNC_UNDEF); /*    */
  else {
    lvartemp = lvartos;  /*   
                              */
    get_args();  /*    */
    temp = prog; /*    */
    func_push(lvartemp);  /*   
                               */
    prog = loc;  /*  prog    */
    get_params(); /*   
                       */
    interp_block(); /*   */
    prog = temp; /*  prog */
    lvartos = func_pop(); /*  
                               */
  }
}

/*     
    . */
void get_args()
{
  int value, count, temp[NUM_PARAMS];
  struct var_type i;

  count = 0;
  get_token();
  if(*token != '(') sntx_err(PAREN_EXPECTED);

  /*    */
  do {
    eval_exp(&value);
    temp[count] = value;  /*   */
    get_token();
    count++;
  }while(*token == ',');
  count--;
  /*   local_var_stack    */
  for(; count>=0; count--) {
    i.value = temp[count];
    i.v_type = ARG_L;
    local_push(i);
  }
}

/*   . */
void get_params()
{
  struct var_type *p;
  int i;

  i = lvartos-1;
  do { /*    */
    get_token();
    p = &local_var_stack[i];
    if(*token != ')' ) {
      if(tok != INT_L && tok != CHAR_L)
        sntx_err(TYPE_EXPECTED);

      p->v_type = token_type;
      get_token();

      /*     ,
               */
      strcpy(p->var_name, token);
      get_token();
      i--;
    }
    else break;
  } while(*token == ',');
  if(*token != ')') sntx_err(PAREN_EXPECTED);
}

/*   . */
void func_ret()
{
  int value;

  value = 0;
  /*   ,    */
  eval_exp(&value);

  ret_value = value;
}

/*   . */
void local_push(struct var_type i)
{
  if(lvartos > NUM_LOCAL_VARS)
    sntx_err(TOO_MANY_LVARS);

  local_var_stack[lvartos] = i;
  lvartos++;
}

/*      . */
int func_pop()
{
  functos--;
  if(functos < 0) sntx_err(RET_NOCALL);
  return call_stack[functos];
}

/*      . */
void func_push(int i)
{
  if(functos>NUM_FUNC)
   sntx_err(NEST_FUNC);
  call_stack[functos] = i;
  functos++;
}

/*   . */
void assign_var(char *var_name, int value)
{
  register int i;

  /*     */
  for(i=lvartos-1; i >= call_stack[functos-1]; i--)  {
    if(!strcmp(local_var_stack[i].var_name, var_name)) {
      local_var_stack[i].value = value;
      return;
    }
  }
  if(i < call_stack[functos-1])
  /*   ,
           */
    for(i=0; i < NUM_GLOBAL_VARS; i++)
      if(!strcmp(global_vars[i].var_name, var_name)) {
        global_vars[i].value = value;
        return;
      }
  sntx_err(NOT_VAR); /*    */
}

/*   . */
int find_var(char *s)
{
  register int i;

  /*    */
  for(i=lvartos-1; i >= call_stack[functos-1]; i--)
    if(!strcmp(local_var_stack[i].var_name, token))
      return local_var_stack[i].value;

  /*    ,
          */
  for(i=0; i < NUM_GLOBAL_VARS; i++)
    if(!strcmp(global_vars[i].var_name, s))
      return global_vars[i].value;

  sntx_err(NOT_VAR); /*    */
  return -1;
}

/*    , 
    1,  0.
*/
int is_var(char *s)
{
  register int i;

  /*    ? */
  for(i=lvartos-1; i >= call_stack[functos-1]; i--)
    if(!strcmp(local_var_stack[i].var_name, token))
      return 1;

  /*   -     */
  for(i=0; i < NUM_GLOBAL_VARS; i++)
    if(!strcmp(global_vars[i].var_name, s))
      return 1;

  return 0;
}

/*   if. */
void exec_if()
{
  int cond;

  eval_exp(&cond); /*  if- */

  if(cond) { /*  -  if- */
    interp_block();
  }
  else { /*     if-
              else-,    */
    find_eob(); /*    */
    get_token();

    if(tok != ELSE_L) {
      putback();  /*  ,
                      else-  */
      return;
    }
    interp_block();
  }
}

/*   while. */
void exec_while()
{
  int cond;
  char *temp;

  putback();
  temp = prog;  /*     while */
  get_token();
  eval_exp(&cond);  /*    */
  if(cond) interp_block();  /*   ,  
                                */
  else {  /*      */
    find_eob();
    return;
  }
  prog = temp;  /*     */
}

/*   do. */
void exec_do()
{
  int cond;
  char *temp;

  putback();
  temp = prog;  /*     */

  get_token(); /*    */
  interp_block(); /*   */
  get_token();
  if(tok != WHILE_L) sntx_err(WHILE_EXPECTED);
  eval_exp(&cond); /*    */
  if(cond) prog = temp; /*   ,
    ,    
     */
}

/*   . */
void find_eob()
{
  int brace;

  get_token();
  brace = 1;
  do {
    get_token();
    if(*token == '{') brace++;
    else if(*token == '}') brace--;
  } while(brace);
}

/*   for. */
void exec_for()
{
  int cond;
  char *temp, *temp2;
  int brace ;

  get_token();
  eval_exp(&cond);  /*   */
  if(*token != ';') sntx_err(SEMI_EXPECTED);
  prog++; /*  ; */
  temp = prog;
  for(;;) {
    eval_exp(&cond);  /*   */
    if(*token != ';') sntx_err(SEMI_EXPECTED);
    prog++; /*  ; */
    temp2 = prog;

    /*     */
    brace = 1;
    while(brace) {
      get_token();
      if(*token == '(') brace++;
      if(*token == ')') brace--;
    }

    if(cond) interp_block();  /*   ,
                                    */
    else {  /*      */
      find_eob();
      return;
    }
    prog = temp2;
    eval_exp(&cond); /*   */
    prog = temp;  /*     */
  }
}
