#include "header.h"
#include <sstream>
#include <fstream>
#include <SDL2/SDL.h>
#include <SDL2/SDL_main.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>

#include "input/mouse.h"
#include "input/keyboard.h"
#include "rect.h"
#include "texture.h"

#include "LittleC/header.h"

///----------------------------------------------------------
///----------------------------------------------------------
///----------------------------------------------------------
#ifdef _WIN32
#include <windows.h>
#endif
///----------------------------------------------------------
///----------------------------------------------------------

bool init();
bool loadMedia();
void close();
bool LoadSysPics();
bool LoadTemplsUnits();
bool LoadTemplsSkills();

//The surface contained by the window
SDL_Surface* gStretchedSurface = NULL;

bool init()
{
  //Load ttf
  if(!fontInit()){cout<<"\nCant load font!";return false;}
  if(!LoadTemplsUnits()){cout<<"\nCant load units!"; return false;}
  LoadTemplsSkills();

	//Initialize SDL
	if(SDL_Init(SDL_INIT_EVERYTHING) < 0)
	{
		printf("SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
		return false;
	}
	else
	{
	  //Set texture filtering to linear
		if( !SDL_SetHint( SDL_HINT_RENDER_SCALE_QUALITY, "1" ) )
		{
			cout<<"\nWarning: Linear texture filtering not enabled!";
			return false;
		}

		//Create window
		gWindow = SDL_CreateWindow("JaG", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN);
		if(gWindow == NULL)
		{
			printf("Window could not be created! SDL_Error: %s\n", SDL_GetError());
			return false;
		}
		else
		{
			//Get window surface
			#ifdef _WIN32
			renderer=SDL_CreateRenderer(gWindow, 2, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);
			#else
			renderer=SDL_CreateRenderer(gWindow, 1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);
			#endif
			if(renderer==NULL)
      {
        cout<<"\nRender is not loaded! SDL_Error: "<<SDL_GetError();
        return false;
      }
      SDL_SetRenderDrawColor( renderer, 0xFF, 0xFF, 0xFF, 0xFF );
		}
	}
  if(!LoadSysPics()){cout<<"Cant load SysPics";return false;}


	return true;
}


bool LoadSettingsCom()
{
    string tmp;
    ifstream fileSet(fileSettingCom.c_str(), ios::in);
    if(!fileSet){cout<<"\nCan't load "<<fileSettingCom; return false;}
    textureImg textureTMP;

    while(getline(fileSet, tmp, '\n'))
    {
        cout<<'\n'<<tmp;
        textureTMP.SetPath(tmp);
        pictures.push_back(textureTMP);
        ///addressPics.push_back(tmp);
    }


    return true;
}

bool loadMedia()
{
	//Load splash image
	for(unsigned int i=0; i<pictures.size(); i++)
    {
        ///pictures[i]=SDL_LoadBMP(addressPics[i].c_str());
        pictures[i].SetTexture(loadTexture(pictures[i].GetPathChar()));
        pictures[i].SetShow(false);
        if(pictures[i].GetTexture()==NULL)
        {
            cout<<"Cant load image: "<<pictures[i].GetPath()<<".   "<<SDL_GetError();
            //SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, "Failure", "Cant load image: "+pictures[i].GetPath()+".   "+SDL_GetError(), NULL);
            return false;
        }
    }

	return true;
}

bool LoadScenary()
{
    return InitLittleC(fileScenary.c_str());
}

void close()
{
	//Deallocate surface
	for(unsigned int i=0; i<pictures.size(); i++)
    {
        SDL_DestroyTexture(pictures[i].GetTexture());
    }
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(gWindow);

    TTF_CloseFont(font);

    TTF_Quit();
    IMG_Quit();
    SDL_Quit();
}

const Uint8* currentKeyStates;
void ButtonChecker()
{
  currentKeyStates = SDL_GetKeyboardState(NULL);

  for(int i=0; i<300; i++)
  {
    if(currentKeyStates[i]){PressedKey(i);}
  }
}

int bos=0;

Uint32 AllTimerTick(Uint32 interval, void* param)
{
  if(timerFunctions.size()==0){return interval;}

  for(timerIterator=timerFunctions.begin(); timerIterator!=timerFunctions.end(); timerIterator++)
  {
    if(timerFunctions.size()==0){return interval;}
    else{timerIterator->second.AddTick();}
  }

  return interval;
}

Uint32 AllUnitTick(Uint32 interval, void* param)
{
  if(units.size()==0){return interval;}

  for(unitsIterator=units.begin(); unitsIterator!=units.end(); unitsIterator++)
  {
    if(units.size()==0){return interval;}
    else{unitsIterator->second.Tick();}
  }

  return interval;
}

void draw()
{
    SDL_RenderClear(renderer);
    //-------------------------------------------------------------------
    for(unsigned int i=0; i<pictures.size(); i++)
    {
        pictures[i].Draw();
    }
    for(gameTextMapIterator=gameTextMap.begin(); gameTextMapIterator!=gameTextMap.end(); gameTextMapIterator++)
    {
      if(gameTextMap.size()==0){break;}
      else{gameTextMapIterator->second.Draw();}
    }
    for(unitsIterator=units.begin(); unitsIterator!=units.end(); unitsIterator++)
    {
      if(units.size()==0){break;}
      else{unitsIterator->second.Draw();}
    }
    //-------------------------------------------------------------------
    for(unsigned int i=0; i<systemPictures.size(); i++)
    {
        systemPictures[i].Draw();
    }
    if(!choosenUnits.empty())
    {
      sysTextMap[0].SetText(ftoa(choosenUnits[0]->GetLives())); sysTextMap[0].SetW(sysTextMap[0].GetLenText()*10);
      sysTextMap[1].SetText(ftoa(choosenUnits[0]->GetMana())); sysTextMap[1].SetW(sysTextMap[1].GetLenText()*10);
    }
    else
    {
      sysTextMap[0].SetText("Health"); sysTextMap[0].SetW(80);
      sysTextMap[1].SetText("Mana"); sysTextMap[1].SetW(60);
    }
    for(unsigned int i=0; i<sysTextMap.size(); i++)
    {
        sysTextMap[i].Draw();
    }
    if(skillDraw!=NULL)
      for(unsigned i=0; i<skillDraw->size(); i++)
      {
        (*skillDraw)[i].Draw();
      }
    //-------------------------------------------------------------------
    SDL_RenderPresent(renderer);
}

void *data;

#ifdef _WIN32
int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
#else
int main(int argc, char* args[])
#endif
{
	//Start up SDL and create window
	if(!init())
	{
		cout<<"\nFailed to initialize!";
	}
	else
	{
		//Loadings
		if(!LoadScenary()){cout<<"\nFailed to load script.txt";}
		else
		{
		    StartLittleC();///Старт запуска скрипта

        bool quit = false;

        currentKeyStates = SDL_GetKeyboardState(NULL);

        SDL_AddTimer(1, AllTimerTick, data);
        SDL_AddTimer(10, AllUnitTick, data);
        //While application is running
        while(!quit)
        {
           //Handle events on queue
           while(SDL_PollEvent(&e) != 0)
           {
             //User requests quit
             switch(e.type)
             {
               case SDL_QUIT:
                 quit = true;
               break;

               case SDL_MOUSEMOTION:
                 MouseMoving();
               break;

               case SDL_MOUSEBUTTONDOWN:
                 if(e.button.button==SDL_BUTTON_LEFT)
                  {mouseDown[0]=true;SetMouseStartSelectRect();}
                 if(e.button.button==SDL_BUTTON_RIGHT)
                 {mouseDown[1]=true;}
               break;

               case SDL_MOUSEBUTTONUP:
                 if(e.button.button==SDL_BUTTON_LEFT)
                  {
                    mouseDown[0]=false; systemPictures[0].SetShow(false);
                    if(!MouseEndChoose()){MouseLeftClick();}
                  }
                 if(e.button.button==SDL_BUTTON_RIGHT)
                  {mouseDown[1]=false; MouseRightClick();}
               break;

               case SDL_KEYDOWN: case SDL_KEYUP:
                 ButtonChecker();
               break;
             }
           }
           draw();
        }
    }
	}

	//Free resources and close SDL
	close();

	return 0;
}

bool LoadSysPics()
{
  textureImg img("SysPics/select.bmp");
  systemPictures.push_back(img);

  if(systemPictures[0].GetTexture()==NULL)
  {
      cout<<"\nCant load SYSimage: "<<systemPictures[0].GetPath()<<".   "<<SDL_GetError();
      e.type=SDL_QUIT;
      return false;
  }


  textureImg img2("SysPics/unitpanelhpmn.bmp");
  systemPictures.push_back(img2);

  if(systemPictures[1].GetTexture()==NULL)
  {
      cout<<"\nCant load SYSimage: "<<systemPictures[1].GetPath()<<".   "<<SDL_GetError();
      e.type=SDL_QUIT;
      return false;
  }
  systemPictures[1].SetCoords(0, 560);
  systemPictures[1].SetShow(true);



  textureImg img3("SysPics/skillpanel.bmp");
  systemPictures.push_back(img3);

  if(systemPictures[2].GetTexture()==NULL)
  {
      cout<<"\nCant load SYSimage: "<<systemPictures[2].GetPath()<<".   "<<SDL_GetError();
      e.type=SDL_QUIT;
      return false;
  }
  systemPictures[2].SetCoords(740, 560);
  systemPictures[2].SetShow(true);
  //--------------------------------------------------------
  gameText txtTmp;
  SDL_Rect rect;
  rect.x=15; rect.y=717; rect.w=100; rect.h=20;

  sysTextMap.push_back(txtTmp);
  sysTextMap[0].SetText("Health");
  sysTextMap[0].SetRect(rect);
  sysTextMap[0].SetHide(false);

  rect.x=15; rect.y=743; rect.w=100; rect.h=20;
  sysTextMap.push_back(txtTmp);
  sysTextMap[1].SetText("Mana");
  sysTextMap[1].SetRect(rect);
  sysTextMap[1].SetHide(false);

  return true;
}

bool LoadTemplsUnits()
{
  UnitTemplate tmpUn;
  tmpUn.templateID=0;
  tmpUn.livesPoints=100.;
  tmpUn.manaPoints=0.;
  tmpUn.picModel=nowLevel+"img/peon.bmp";
  tmpUn.radiusPath=5;
  tmpUn.speedBasic=10;
  tmpUn.speedMax=20;
  tmpUn.canRun=true;

  templUnits.push_back(tmpUn);


  return true;
}

bool LoadTemplsSkills()
{
  skillMagicList[0].name="Simple skill";
  skillMagicList[0].aim=0;
  skillMagicList[0].timeCasting=10;
  skillMagicList[0].model="levels/level1/img/simpleskill.bmp";


  return true;
}



